/**
 * Represents a financial position in a portfolio.
 */
export class Position {
    /**
     * The symbol name of the asset
     */
    readonly symbol: string;

    /**
     * The total number of shares, not including open orders
     */
    readonly quantity: number;

    /**
     * The total dollar amount of the position
     */
    readonly marketValue: number;

    /**
     * The total cost basis
     */
    readonly costBasis: number;

    /**
     * The current asset price per share
     */
    readonly pricePerShare: number;


    /**
     * Creates a new Portfolio object.
     * @constructor
     * @param {string} symbol - The symbol of the asset in the portfolio.
     * @param {number} quantity - The quantity of the asset in the portfolio.
     * @param {number} marketValue - The market value of the asset in the portfolio.
     * @param {number} costBasis - The cost basis of the asset in the portfolio.
     * @param {number} pricePerShare - The price per share of the asset in the portfolio.
     */
    constructor(symbol: string, quantity: number, marketValue: number, costBasis: number, pricePerShare: number) {
        this.symbol = symbol;
        this.quantity = quantity;
        this.marketValue = marketValue;
        this.costBasis = costBasis;
        this.pricePerShare = pricePerShare;
    }
}

/**
 * Represents a portfolio of financial positions.
 */
export class Portfolio {
    /**
     * An array of positions held in the portfolio.
     */
    readonly positions: Position[];

    /**
     * Creates a new Portfolio instance.
     * @constructor
     * @param {Position[]} positions - An array of Position objects representing the positions in the portfolio.
     */
    constructor(positions: Position[]) {
        this.positions = positions;
    }
}

/**
 * A provider for fetching a portfolio.
 */
export interface PortfolioProvider {

    /**
     * Fetches the portfolio.
     * @returns A promise that resolves with the fetched portfolio.
     */
    readonly fetchPortfolio: () => Promise<Portfolio>;
}
